local knifeHeight = 240
local knifeWidth = 26
local maxKnifes = 40
local wheelRadius = 175
local whellDistance = 190
local knifeStartDistance = 400
local knifeStickDistance = wheelRadius
local maxParticles = 5
local bouncePower = 18
local bounceDuration = 0.3

local STATES = {GAME = 1, RESET = 2, GAMEOVER = 3}

GameScene = Core.class(Sprite)

function GameScene:init()
	self.cam = Camera.new("bg", "knifes", "wheel", "particles", "mini", "ui")
	self:addChild(self.cam)
	
	self.world = SCD.new(32)
	
	self.wheel = Wheel.new(self.world, ScreenW / 2, ScreenH / 2 - whellDistance, wheelRadius)
	
	self.particles = Particles.new()
	self.spreadVec = Vec2.new()
	self.cam:add("particles", self.particles)
	
	self.gameState = STATES.GAME
	self.lastKnife = nil
	self.spin = true
	self.spinSpeed = -80
	self.curKnife = 1
	self.maxKnifes = maxKnifes -- per level
	self.knifes = {}		-- all knifes
	self.pooledKnifes = {}	-- updated knifes
	
	self:createKnifes()
	self:createLevel()
	
	self.cam:add("wheel", self.wheel)
	
	-- DEBUG DRAW --
	self.console = Console.new(10)
	self.console:setPosition(20, ScreenH - 20)
	self.console:setScale(2, 2)
	self.cam:add("ui", self.console)
	
	self:addEventListener(Event.TOUCHES_BEGIN, self.touchBegin, self)
	self:addEventListener(Event.ENTER_FRAME, self.enterFrame, self)
	self:addEventListener(Event.KEY_UP, function(e)
		if (e.keyCode == KeyCode.R) then
			self:reset()
		elseif (e.keyCode == KeyCode.X) then
			self.spinSpeed *= -1
		elseif (e.keyCode == KeyCode.D) then
			self.boss:shoot()
		elseif (e.keyCode == KeyCode.T) then
			for k,v in ipairs(self.knifes) do
				v:throw()
			end
			self.gameState = STATES.GAMEOVER
		elseif (e.keyCode == KeyCode.S) then
			self.spin = false
		end
	end)
	self:addEventListener(Event.KEY_DOWN, function(e)
		if (e.keyCode == KeyCode.S) then
			self.spin = true
		end
	end)
	
	local t = Timer.new(1500)
	t:start()
	t:addEventListener(Event.TIMER, function()
		if (math.random() >= 0.5) then
			self.spinSpeed *= -1
		end
	end)
	
	local dir = 1
	local t2 = Timer.new(1000)
	t2:start()
	t2:addEventListener(Event.TIMER, function()
		if (math.random() >= 0.5) then
			self.boss:shoot(dir)
			dir *= -1
		end
	end)
end
function GameScene:createKnifes()
	for i = 1, self.maxKnifes do
		local k = Knife.new(self.world,
			ScreenW / 2, 
			ScreenH / 2 + knifeStartDistance + (i > 1 and knifeHeight / 2 or 0), 
			knifeWidth, 
			knifeHeight
		)
		k:setAlpha(i > 1 and 0 or 1)
		self.cam:add("knifes", k)
		
		rawset(self.knifes, i, k)
	end
end

function GameScene:createLevel()
	local lr = self.cam:getLayer("wheel")
	local c = 4
	local step = 360 / c
	local v = 0.9
	
	for i = 1, c do
	self.wheel:addObstacle({
		name = "box", 
		dist = wheelRadius, 
		extraRotation = 10, 
		angle = step * i, 
		w = 150, h = 32, 
		--faceTo = false, 
		container = lr
	})
	v+=0.02
	end
	self.wheel:addObstacle({
		name = "circle", 
		dist = wheelRadius + 80, 
		rotationScale = 0.5,
		angle = 180, 
		r = 32, 
		container = lr
	})
	
	self.boss = MiniBoss.new(self.world, self.cam:getLayer("particles"))
	self.boss:setPosition(ScreenW / 2, 20)
	self.cam:add("mini", self.boss)
end
function GameScene:resetKnife(i)
	local knife = self.knifes[i]
	knife:stopAnimation()
	knife.moving = false
	knife.sticked = false
	knife.falling = false
	knife:setPosition(ScreenW / 2, ScreenH / 2 + knifeStartDistance + (i > 1 and knifeHeight / 2 or 0))
	knife:setAlpha(i > 1 and 0 or 1)
	knife:faceTo(self.wheel:getShape().pos)
	
	self.curKnife = 1
end
function GameScene:reset()
	self.gameState = STATES.RESET
	if (self.lastKnife) then
		self.lastKnife:setAlpha(0)
		self.lastKnife = nil
	end
	self.spin = false
	local l = #self.pooledKnifes
	for i = l, 1, -1 do
		table.remove(self.pooledKnifes, i)
	end
	for k,v in ipairs(self.knifes) do
		self:resetKnife(k)
	end
	self.boss:free()
	self.cam:add("wheel", self.wheel)
	self.spin = true
	self.gameState = STATES.GAME
end
function GameScene:gameOver()
	self.cam:add("wheel", self.lastKnife, 0)
	self.lastKnife:throw()
	self.gameState = STATES.GAMEOVER
	
	Timer.delayedCall(1200, self.reset, self)
end
function GameScene:fireParticles(y)
	local l = self.cam:getLayer("wheel")
	l:stopAnimation()
	l:setPosition(0, -bouncePower)
	l:animate(bounceDuration, {y = 0}, easing.outCubic)
	
	for i = 1, maxParticles do
		self.spreadVec:set(randomFloat(0.2, 1) * 80, 0)
		self.spreadVec:rotate(^<(math.random(45, 135)+180))
		t = self.particles:addParticles(self.wheel:getX(),self.wheel:getY() + self.wheel.radius,10,0,200)
		self.particles:setParticleSpeed(t, self.spreadVec.x, self.spreadVec.y, 2.1)
		self.particles:setParticleDecay(t, 0.96,0.93)--decayAlpha, decayGrowth, decayAngular)
		self.particles:setParticleColor(t, 0xDED6C0)
	end
end
function GameScene:wheelGotHit(knife)
	knife.moving = false
	knife.sticked = true
	y = self.wheel:getY() + knifeStickDistance
	knife:setY(y)
	self:fireParticles(y)
end
function GameScene:updateKnife(dt, knife)
	
	if (self.spin) then
		knife:updateRotation(dt, self.wheel:getShape().pos, self.spinSpeed)
	end
	if (knife.moving) then
		local x, y = knife:getPosition()
		y += knife.speed * dt
		knife:setY(y)
		local list = self.world:check(knife:getShape())
		for _,v in ipairs(list) do
		
			if (v.name == "wheel" and knife.moving) then
				self:wheelGotHit(knife)
			elseif ((v.name == "obstacle") or (v.name == "shuriken") or (v.name == "knife" and v.ptr.sticked)) then
				self.lastKnife = knife
				self:gameOver()
			end
		end
	end
end

function GameScene:enterFrame(e)
	local dt = e.deltaTime
	if (self.gameState == STATES.GAME) then
		self.boss:update(dt)
		if (self.spin) then
			--self.wheel:setRotation(self.wheel:getRotation() + self.spinSpeed * dt)
			self.wheel:rotate(self.spinSpeed * dt)
		end
		
		for _,v in pairs(self.pooledKnifes) do
			self:updateKnife(dt, v)
		end
		
	elseif (self.gameState == STATES.GAMEOVER) then
		self.lastKnife:updateFall(dt)
	end
end

function GameScene:shoot(knife)
	knife.moving = true
	knife:setPosition(ScreenW / 2, ScreenH / 2 + knifeStartDistance)
	knife:stopAnimation()
	self.cam:add("wheel", knife)
	self.cam:add("wheel", self.wheel)
	rawset(self.pooledKnifes, #self.pooledKnifes + 1, knife)
	--self.pooledKnifes[#self.pooledKnifes + 1] = knife
end
function GameScene:touchBegin(e)
	if (self.gameState == STATES.GAME and self.curKnife <= self.maxKnifes) then
		self:shoot(self.knifes[self.curKnife])
		self.curKnife += 1
		if (self.curKnife <= self.maxKnifes) then
			self.knifes[self.curKnife]:animate(0.2, {alpha = 1, y = ScreenH / 2 + knifeStartDistance}, easing.outCubic)
		end
	end
end