Layer = Core.class(Sprite)

function Layer:init(name)
	self.name = name or ""
end

function Layer:animate(duration, target, ease)
	if (self.gt) then
		self.gt:setValues(target)
		self.gt:setDuration(duration)
		self.gt:setEase(ease)
		self.gt:setPaused(false)
	else
		ease = ease or easing.linear
		self.gt = GTween.new(self, duration, target, {ease=ease})
	end
end

LayersHandler = Core.class(Sprite)

function LayersHandler:init(...)
	self.names = {...}
	self.layers = {}
	for i = 1, #self.names do
		local name = self.names[i]
		self.layers[name] = Layer.new(name)
		self:addChild(self.layers[name])
	end
end

function LayersHandler:addLayer(name)
	local l = Layer.new(name)
	self.names[#self.names] = name
	self.layers[name] = l
	self:addChild(l)
end

function LayersHandler:addLayers(...)
	local names = {...}
	for i = 1, #names do self:addLayer(names[i]) end
end

function LayersHandler:removeLayer(name)
	for i = 1, #self.names do
		local ename = self.names[i]
		if (name == ename) then
			self.names[i] = nil
			self:removeChild(self.layers[name])
		end
	end
end

function LayersHandler:removeLayers(...)
	local names = {...}
	for i = 1, #names do self:removeLayer(names[i]) end
end

function LayersHandler:add(name, sprite)
	if not self.layers[name]:contains(sprite) then
		self.layers[name]:addChild(sprite)
	end
end

function LayersHandler:remove(name, sprite)
	if self.layers[name]:contains(sprite) then
		self.layers[name]:removeChild(sprite)
		return true
	end
	return false
end

function LayersHandler:animate(name, duration, target, ease) self.layers[name]:animate(duration, target, ease) end

function LayersHandler:setLayerVisible(flag) self.layers[name]:setVisible(flag) end

function LayersHandler:setLayerBlendMode(name, m) self.layers[name]:setBlendMode(m) end

function LayersHandler:setLayerColorTransform(name, color) self.layers[name]:setColorTransform(color[1]/255, color[2]/255, color[3]/255, 1) end

function LayersHandler:setLayerAlpha(name, val) self.layers[name]:setAlpha(val) end

---------

function LayersHandler:getLayer(name) return self.layers[name] end