local min,max=math.min,math.max
TCircle = Core.class()
function TCircle:init(x, y, r) 
	self.pos = Vec2.new(x or 0, y or 0)
	self.r = r or 1 
	self.vertices = {}
	self:recalcVertices()
end

function TCircle:setPosition(x, y) self.pos.x = x;self.pos.y = y self:recalcVertices() end
function TCircle:setRadisu(r) self.r = r self:recalcVertices() end
function TCircle:setX(x) self.pos.x = x self:recalcVertices() end
function TCircle:setY(y) self.pos.y = y self:recalcVertices() end

function TCircle:getPosition() return self.pos end
function TCircle:getRadius() return self.r end
function TCircle:getX() return self.pos.x end
function TCircle:getY() return self.pos.y end
function TCircle:recalcVertices() 
	self.vertices = {self.pos.x - self.r,self.pos.y - self.r,	
		self.pos.x + self.r,self.pos.y - self.r,	
		self.pos.x + self.r,self.pos.y + self.r,
		self.pos.x - self.r,self.pos.y + self.r
	}	
end
function TCircle:getAABB() return self.vertices end
function TCircle:path2d() return CreateCircle(self.r) end

TRect = Core.class()
function TRect:init(x, y, w, h) 
	self.pos = Vec2.new(x or 0, y or 0)
	self.size = Vec2.new(w or 1, h or 1)
	self.rotation = 0
	self.prevRot = 0
	self.vertices = {}
	self.points = {}
	self:updatePoints()
	self:recalcVertices()
end

function TRect:setRotation(angle) self.prevRot = self.rotation self.rotation = angle self:recalcVertices()  end -- in DEG
function TRect:setPosition(x, y) self.pos.x = x;self.pos.y = y self:updatePoints() self:recalcVertices() end
function TRect:setSize(w, h) self.size.x = w;self.size.y = h self:updatePoints() end
function TRect:setWidth(w) self.size.x = w self:updatePoints() end
function TRect:setHeight(h) self.size.y = h self:updatePoints() end
function TRect:setX(x) self.pos.x = x self:updatePoints() self:recalcVertices() end
function TRect:setY(y) self.pos.y = y self:updatePoints() self:recalcVertices() end

function TRect:getRotation() return self.rotation end -- in DEG
function TRect:getRotationRad() return ^<self.rotation end -- in RAD
function TRect:getPosition() return self.pos end
function TRect:getSize() return self.size end
function TRect:getWidth() return self.size.w end
function TRect:getHeight() return self.size.h end
function TRect:getX() return self.pos.x end
function TRect:getY() return self.pos.y end
function TRect:updatePoints()
	self.vertices[1] = Vec2.new(self.pos.x - self.size.x / 2, self.pos.y - self.size.y / 2)
	self.vertices[2] = Vec2.new(self.pos.x + self.size.x / 2, self.pos.y - self.size.y / 2)
	self.vertices[3] = Vec2.new(self.pos.x + self.size.x / 2, self.pos.y + self.size.y / 2)
	self.vertices[4] = Vec2.new(self.pos.x - self.size.x / 2, self.pos.y + self.size.y / 2)
end
function TRect:recalcVertices()
	local delta = ^<(self.rotation - self.prevRot)
	self.vertices[1]:rotateAround(delta, self.pos)
	self.vertices[2]:rotateAround(delta, self.pos)
	self.vertices[3]:rotateAround(delta, self.pos)
	self.vertices[4]:rotateAround(delta, self.pos)
	self.topLeft = {
		x=min(self.vertices[1].x,self.vertices[2].x,self.vertices[3].x,self.vertices[4].x),
		y=min(self.vertices[1].y,self.vertices[2].y,self.vertices[3].y,self.vertices[4].y)
	}
	self.bottomRight = {
		x=max(self.vertices[1].x,self.vertices[2].x,self.vertices[3].x,self.vertices[4].x),
		y=max(self.vertices[1].y,self.vertices[2].y,self.vertices[3].y,self.vertices[4].y)
	}
	
	self.points = {
		self.topLeft.x,
		self.topLeft.y,
		
		self.bottomRight.x,
		self.topLeft.y,
		
		self.bottomRight.x,
		self.bottomRight.y,
		
		self.topLeft.x,
		self.bottomRight.y
	}
	
end
function TRect:getAABB() return self.points end
function TRect:path2d() 
	local w, h = self.size:unpack()
	local r = CreateRect(w,h) 
	r:setAnchorPosition(w/2, h/2)
	return r
end
