local thick = 8

Wheel = Core.class(Sprite)

function Wheel:init(world, x, y, radius)
	self.radius = radius
	self.collider = Circle.new("wheel", world, radius)
	self.collider:setPtr(self)
	self:setPosition(x, y)
	
	self.obstacles = {}
	
	self:createGraphics()
end

function Wheel:getShape() return self.collider.shape end

function Wheel:createGraphics()
	local r = self.collider.shape:getRadius()
	self.gr = CreateCircle(r)
	self.gr:setFillColor(0xDED6C0)
	self.gr:setLineThickness(thick)
	self.gr:setLineColor(0xAF685D)
	self:addChild(self.gr)
	
	local ln = CreateRect(r - thick / 2, thick)
	ln:setFillColor(0xD1A579, 1)
	ln:setLineColor(0,0)
	self:addChild(ln)
end

function Wheel:setPosition(x, y)
	self.collider:setPosition(x, y)
	Sprite.setPosition(self, x, y)
end

function Wheel:rotate(amount)
	Sprite.setRotation(self, self:getRotation() + amount)
	
	local x, y = self:getPosition()
	local pos = {x=x, y=y}
	local arad = ^<amount
	for i,o in ipairs(self.obstacles) do
		o.body:rotateAround(^<(amount * o.rotationScale), x, y)
		o.gfx:setPosition(o.body:getPosition())
		if (o.name ~= "circle" and o.faceTo) then
			o.body:faceTo(pos, o.rotation)
			o.gfx:setRotation(o.body:getRotation())
		end
	end
end

-- params:
-- name				- box or circle
-- dist				- how far body is located
-- extraRotation	- rotate body on that amount 
-- angle			- where to put obstacle
-- w, h				- rect size (or r - radius for circle)
-- faceTo
-- container
function Wheel:addObstacle(params)
	local a = ^<params.angle
	local r = params.dist or self.collider.shape.r
	local x = self:getX() + math.cos(a) * r
	local y = self:getY() + math.sin(a) * r
	
	local body
	local g = params.gfx
	if (params.faceTo == nil) then params.faceTo = true end
	
	if (params.name == "box") then
		local w, h = params.w, params.h
		body = Box.new("obstacle", self.collider.world, w, h)
		if (not g) then
			g = CreateRect(w, h, 0.5, 0.5)
			g:setFillColor(0xDED6C0)
			g:setLineThickness(thick)
			g:setLineColor(0xAF685D)
			g:setPosition(x, y)
		end
	elseif (params.name == "circle") then
		local r = params.r
		body = Circle.new("obstacle", self.collider.world, r)
		if (not g) then
			g = CreateCircle(r)
			g:setFillColor(0xDED6C0)
			g:setLineThickness(thick)
			g:setLineColor(0xAF685D)
			g:setPosition(x, y)
		end
	end	
	
	body:setPosition(x, y)
	body:setPtr(self)
	local t = {
		name = params.name, body = body, gfx = g, 
		faceTo = params.faceTo, startAngle = params.angle,
		rotation = params.extraRotation or 0,
		rotationScale = params.rotationScale or 1
	}
	rawset(self.obstacles, #self.obstacles + 1, t)
	params.container:addChild(g)
end